 {
******************************************************
  USB Disk Ejector
  Copyright (c) 2006, 2007, 2008 Ben Gorman
  Http://quick.mixnmojo.com
******************************************************
}
{
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}

unit formOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, ExtCtrls,
  JvExComCtrls, JvHotKey,
  uVistaFuncs, uDiskEjectOptions, uCustomHotKeyManager, uDiskEjectConst;

type
  TOptionsfrm = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    GroupBox1: TGroupBox;
    CheckBox1: TCheckBox;
    chkBoxShowWindowsEject: TCheckBox;
    GroupBox3: TGroupBox;
    chkboxCloseToTray: TCheckBox;
    chkboxShowBalloonMessages: TCheckBox;
    chkboxMinimizeToTray: TCheckBox;
    GroupBox4: TGroupBox;
    chkboxSaveWindowPosition: TCheckBox;
    chkboxSaveWindowSize: TCheckBox;
    chkboxStartAppMinimized: TCheckBox;
    GroupBox5: TGroupBox;
    comboBoxHotKeyAction: TComboBox;
    hotKey1: TJvHotKey;
    btnAddHotKey: TBitBtn;
    btnRemoveHotKey: TBitBtn;
    comboboxDriveLetter: TComboBox;
    listViewHotkeys: TListView;
    comboboxDriveName: TComboBox;
    lblNoSaveMode: TLabel;
    chkBoxAutosizeWindow: TCheckBox;
    radiogroupAfterEject: TRadioGroup;
    GroupBox2: TGroupBox;
    CheckBox4: TCheckBox;
    chkboxCardPolling: TCheckBox;
    radiogroupCloseApps: TRadioGroup;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure btnAddHotKeyClick(Sender: TObject);
    procedure btnRemoveHotKeyClick(Sender: TObject);
    procedure comboBoxHotKeyActionChange(Sender: TObject);
    procedure listViewHotkeysChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure listViewHotkeysDblClick(Sender: TObject);
    procedure chkBoxAutosizeWindowClick(Sender: TObject);
    procedure chkboxSaveWindowSizeClick(Sender: TObject);
  private
    procedure PopulateDriveLettersComboBox;
    procedure PopulateDriveNamesComboBox;
    procedure UpdateListViewHotKeys;
    procedure AddListviewItem(fListView: TListView; fCaption : string; fSubItems : Array of string);
  public
    { Public declarations }
  end;

var
  Optionsfrm: TOptionsfrm;

implementation

uses formAbout, formMain;

{$R *.dfm}

procedure TOptionsfrm.FormCreate(Sender: TObject);
var
  i: integer;
begin
  //Set this before changing fonts below
  lblNoSaveMode.Font.Style:=[fsBold];

  //Set vista fonts if necessary
  SetDesktopIconFonts(Self.Font);
  if IsWindowsVista then
    Self.Font.size:=self.font.size -1;

  PopulateDriveLettersComboBox;
  PopulateDriveNamesComboBox;
  //listViewHotkeys.Columns[0].Width:=-1;
  //listViewHotkeys.Columns[1].Width:=-1;
  //listViewHotkeys.Columns[1].autosize:=true;
  //listViewHotkeys.Columns[2].Width:=-1;

  Options.HotKeys:=HotKeys;
  Options.RebuildHotKeys;

  if HotKeys.HotKeys.Count = 0 then exit;

  for I := 0 to HotKeys.HotKeys.Count - 1 do
  begin
      case TCustomHotKey(HotKeys.HotKeys[i]).HotKeyType of
        RestoreApp:           AddListviewItem(listViewHotkeys, str_Hotkey_Restore_Window, ['',HotKeys.HotKeyToText(TCustomHotKey(HotKeys.HotKeys[i]).HotKey, true)]);
        EjectByDriveLetter:   AddListviewItem(listViewHotkeys, str_Hotkey_Eject_Letter, [TCustomHotKey(HotKeys.HotKeys[i]).HotKeyParam, HotKeys.HotKeyToText(TCustomHotKey(HotKeys.HotKeys[i]).HotKey, true)]);
        EjectByDriveName:     AddListviewItem(listViewHotkeys, str_Hotkey_Eject_Name,   [TCustomHotKey(HotKeys.HotKeys[i]).HotKeyParam, HotKeys.HotKeyToText(TCustomHotKey(HotKeys.HotKeys[i]).HotKey, true)]);
      end;
  end;



end;

procedure TOptionsfrm.FormShow(Sender: TObject);
begin
  with Options do
  begin
    chkBoxShowWindowsEject.Checked    := ShowWindowsEjectMessage;
    chkboxSaveWindowPosition.Checked  := PreserveWindowLocation;
    chkboxSaveWindowSize.Checked      := PreserveWindowSize;
    chkBoxAutosizeWindow.Checked      := AutoResize;
    chkboxStartAppMinimized.Checked   := StartAppMinimised;
    chkboxCloseToTray.Checked         := CloseToTray;
    chkboxShowBalloonMessages.Checked := BalloonMessages;
    chkboxMinimizeToTray.Checked      := MinimizeToTray;
    chkboxCardPolling.Checked         := CardPolling;
    radiogroupAfterEject.ItemIndex    := AfterEject;

    if (CloseRunningApps = true) and (ForceAppsClose = false) then
      radioGroupCloseApps.ItemIndex:=1
    else
    if (CloseRunningApps = true) and (ForceAppsClose = true) then
      radioGroupCloseApps.ItemIndex:=2
    else
      radioGroupCloseApps.ItemIndex:=0;
    
  end;

  if Options.CommandLine_NoSave then
    lblNoSaveMode.Visible:=true
  else
    lblNoSaveMode.Visible:=false;
end;

procedure TOptionsfrm.listViewHotkeysChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  UpdateListViewHotKeys;
end;

procedure TOptionsfrm.listViewHotkeysDblClick(Sender: TObject);
begin
  if listViewHotKeys.ItemIndex <> -1 then
    ShowMessage(listViewHotkeys.Items[listViewHotKeys.ItemIndex].Caption +
      ': ' +
      listViewHotkeys.Items[listViewHotKeys.ItemIndex].SubItems.Strings[0] +
      #13 + str_Hotkey +
      listViewHotkeys.Items[listViewHotKeys.ItemIndex].SubItems.Strings[1]
    );
end;

procedure TOptionsfrm.PopulateDriveLettersComboBox;
const
  MAX_DRIVES = 26;
var
  DriveName: string;
  i: integer;
begin
  DriveName := 'A:';

  for i := 0 to MAX_DRIVES - 1 do
  begin
    DriveName[1] := 'A';
    Inc(DriveName[1], I);
    comboboxDriveLetter.Items.Add(DriveName);
  end;

  comboboxDriveLetter.ItemIndex:=0;
end;

procedure TOptionsfrm.PopulateDriveNamesComboBox;
var
  i: integer;
begin
  if Ejector.DrivesCount = 0 then exit;

  for I := 0 to Ejector.DrivesCount - 1 do
  begin
    comboboxDriveName.Items.Add(Ejector.RemovableDrives[i].VendorId + ' ' + Ejector.RemovableDrives[i].ProductID);
  end;

end;

procedure TOptionsfrm.UpdateListViewHotKeys;
begin
  if listViewHotkeys.Items.Count > 0 then
    btnRemoveHotkey.Visible := true
  else
    btnRemoveHotkey.Visible := false;
end;

procedure TOptionsfrm.AddListviewItem(fListView: TListView; fCaption : string; fSubItems : Array of string);
var
  i : Integer;
  listitem: tlistitem;
begin
  ListItem := Tlistview(fListView).Items.Add;
  ListItem.Caption := fCaption;
  for i := 0 to High(fSubItems) do
    ListItem.SubItems.Add(fSubItems[i]);
end;

procedure TOptionsfrm.btnAddHotKeyClick(Sender: TObject);
var
  TempHotKeyAction: TCustomHotkeyAction;
  HotKeyParam: string;
begin
  if hotkey1.HotKey = 0 then exit;
  if (comboboxHotkeyAction.ItemIndex = 2) and (comboboxDriveName.Text='') then
  begin
    ShowMessage(str_Hotkey_NoDrive_Name);
    exit;
  end;

  HotKeyParam:='';

  case comboboxHotkeyAction.ItemIndex of
    0: TempHotKeyAction:=RestoreApp;
    1: begin
        TempHotKeyAction:=EjectByDriveLetter;
        HotKeyParam:=comboboxDriveLetter.Items[comboboxDriveLetter.ItemIndex][1];
       end;
    2: begin
        TempHotKeyAction:=EjectByDriveName;
        HotKeyParam:=comboboxDriveName.Text;
       end
  else
       TempHotKeyAction:=RestoreApp;
  end;

  try
    if formMain.HotKeys.AddHotKey(HotKey1.HotKey, TempHotKeyAction, HotKeyParam) = false then
      ShowMessage('Couldnt assign hotkey')
    else
    begin
      case TCustomHotKey(HotKeys.HotKeys[listViewHotkeys.Items.Count]).HotKeyType of
        RestoreApp:           AddListviewItem(listViewHotkeys, str_Hotkey_Restore_Window, ['',HotKeys.HotKeyToText(HotKey1.HotKey, true)]);
        EjectByDriveLetter:   AddListviewItem(listViewHotkeys, str_Hotkey_Eject_Letter, [TCustomHotKey(HotKeys.HotKeys[listViewHotkeys.Items.Count]).HotKeyParam, HotKeys.HotKeyToText(HotKey1.HotKey, true)]);
        EjectByDriveName:     AddListviewItem(listViewHotkeys, str_Hotkey_Eject_Name,   [TCustomHotKey(HotKeys.HotKeys[listViewHotkeys.Items.Count]).HotKeyParam, HotKeys.HotKeyToText(HotKey1.HotKey, true)]);
      end;

    end;
  except on E: EHotKeyAlreadyExists do
    ShowMessage(E.Message);
  end;
end;

procedure TOptionsfrm.btnRemoveHotKeyClick(Sender: TObject);
begin
  if listViewHotkeys.Items.Count = 0 then exit;
  if listViewHotkeys.ItemIndex =-1 then exit;

  if HotKeys.RemoveHotKeyByListIndex(listViewHotkeys.ItemIndex)= false then
    Showmessage(str_Hotkey_Remove_Error)
  else
    listViewHotkeys.Items.Delete(listViewHotkeys.ItemIndex);

  UpdateListviewHotKeys;
end;

procedure TOptionsfrm.chkBoxAutosizeWindowClick(Sender: TObject);
begin
  if chkBoxAutosizeWindow.Checked then chkboxSaveWindowSize.Checked:=false;
end;

procedure TOptionsfrm.chkboxSaveWindowSizeClick(Sender: TObject);
begin
  if chkboxSaveWindowSize.Checked then chkBoxAutosizeWindow.Checked:=false;
end;

procedure TOptionsfrm.comboBoxHotKeyActionChange(Sender: TObject);
begin
  case comboBoxHotKeyAction.ItemIndex of
    0:  begin
          comboboxDriveLetter.Visible:=false;
          comboboxDriveName.Visible:=false;
        end;
    1:  begin
          comboboxDriveLetter.Visible:=true;
          comboboxDriveName.Visible:=false;
        end;
    2: begin
          comboboxDriveLetter.Visible:=false;
          comboboxDriveName.Visible:=true;
        end;
  end;
end;

procedure TOptionsfrm.FormClose(Sender: TObject; var Action: TCloseAction);
begin 
  if ModalResult <> mrOk then
    Exit;
  with Options do
  begin
    ShowWindowsEjectMessage := chkBoxShowWindowsEject.Checked;


    PreserveWindowLocation  := chkboxSaveWindowPosition.Checked;
    PreserveWindowSize      := chkboxSaveWindowSize.Checked;
    AutoResize              := chkBoxAutosizeWindow.Checked;
    StartAppMinimised       := chkboxStartAppMinimized.Checked;
    CloseToTray             := chkboxCloseToTray.Checked;
    BalloonMessages         := chkboxShowBalloonMessages.Checked;
    MinimizeToTray          := chkboxMinimizeToTray.Checked;
    CardPolling             := chkboxCardPolling.Checked;
    AfterEject              := radiogroupAfterEject.ItemIndex;

    if radioGroupCloseApps.ItemIndex = 0 then
    begin
      CloseRunningApps:=false;
      ForceAppsClose:=false
    end
    else
    if radioGroupCloseApps.ItemIndex = 1 then
    begin
      CloseRunningApps:=true;
      ForceAppsClose:=false
    end
    else    
    if radioGroupCloseApps.ItemIndex = 2 then
    begin
      CloseRunningApps:=true;
      ForceAppsClose:=true;
    end;

        //ShowMessage(TCustomHotKey(Options.HotKeysArray[0]).HotKeyParam);
        //ShowMessage(TCustomHotKey(Options.HotKeysArray[1]).HotKeyParam);
  {Options.AHotkey.HotKey:=TCustomHotKey(hotkeys.HotKeys[0]).HotKey;
  Options.AHotkey.HotKeyAtom:=TCustomHotKey(hotkeys.HotKeys[0]).HotKeyAtom;
  Options.AHotkey.HotKeyType:=TCustomHotKey(hotkeys.HotKeys[0]).HotKeyType;
  Options.AHotkey.HotKeyParam:=TCustomHotKey(hotkeys.HotKeys[0]).HotKeyParam;}

  SaveConfig;
 end;

  Ejector.CardPolling:=chkboxCardPolling.Checked;

 //need to queue or delete hotkeys here
end;

end.
