 {
******************************************************
  USB Disk Ejector
  Copyright (c) 2006, 2007, 2008 Ben Gorman
  Http://quick.mixnmojo.com
******************************************************
}
{
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}

//TODO - rewrite with tlist instead of arrays

unit uDriveEjector;

interface

uses
  Classes, Windows, SysUtils, extctrls,
  CfgMgr32, SetupApi, JwaWinBase, Cfg, JwaWinIoctl, jclsysinfo,
  uProcessAndWindowUtils, uDiskEjectConst;

type
  TRemovableDrive = packed record
    DriveLetter: char;
    VolumeName: string;
    VendorId: string;
    ProductID: string;
    ProductRevision: string;
    IsCardReader: boolean;
    CardMediaPresent: boolean;
    BusType: integer;
    ParentDevInst: integer;
  end;

  TDriveEjector = class
  private
    PollTimer: TTimer;
    FOnCardMediaChanged: TNotifyEvent;
    FPollTimerInterval: cardinal;
    FPolling: boolean;

    function GetDrivesCount: integer;
    function GetDrivesDevInstByDeviceNumber(DeviceNumber: Integer; DriveType: UINT; szDosDeviceName: PCHAR): DEVINST;
    function EjectDevice(DriveLetter: string; var EjectErrorCode: integer; ShowEjectMessage: boolean = false): boolean;
    function EjectCard(DriveLetter: string; var EjectErrorCode: integer): boolean;
    function GetParentDriveDevInst(DriveLetter: string; var ParentInstNum: integer): boolean;
    function GetNoDevicesWithSameParentInst(ParentDevInst: integer): integer;
    function GetNoDevicesWithSameProductId(ProductId: string): integer;
    function CheckIfDriveHasMedia(DriveLetter: string): boolean;
    function GetCardPolling: boolean;


    procedure SetCardPolling(Value: boolean);
    procedure FindRemovableDrives;
    procedure ScanDrive(DriveLetter: Char);
    procedure DeleteArrayItem(const Index: Integer);
    procedure CheckForCardReaders;
    procedure OnTimer (Sender:TObject);
  public
    RemovableDrives: array of TRemovableDrive;
    constructor Create;
    destructor Destroy; override;
    //function RemoveDrive(DriveLetter: string; ShowEjectMessage: boolean = false; CardEject: boolean = false): boolean; overload;
    function RemoveDrive(DriveLetter: string; var EjectErrorCode: integer; ShowEjectMessage: boolean = false; CardEject: boolean = false; CloseRunningApps: boolean = false; ForceRunningAppsClosure: boolean = false): boolean; overload;
    procedure RescanAllDrives;

    property DrivesCount: integer read GetDrivesCount;
    property OnCardMediaChanged: TNotifyEvent read FOnCardMediaChanged write FOnCardMediaChanged;
    property CardPollingInterval: cardinal read FPollTimerInterval write FPollTimerInterval;
    property CardPolling: boolean read GetCardPolling write SetCardPolling;
  end;

implementation

procedure TDriveEjector.DeleteArrayItem(const Index: Integer) ;
begin
   if Index > High(RemovableDrives) then Exit;
   if Index < Low(RemovableDrives) then Exit;
   if Index = High(RemovableDrives) then
   begin
     SetLength(RemovableDrives, Length(RemovableDrives) - 1) ;
     Exit;
   end;
   Finalize(RemovableDrives[Index]) ;
   System.Move(RemovableDrives[Index +1], RemovableDrives[Index],(Length(RemovableDrives) - Index -1) * SizeOf(string) + 1) ;
   SetLength(RemovableDrives, Length(RemovableDrives) - 1) ;
end;

constructor TDriveEjector.Create;
begin
  LoadSetupApi;
  LoadConfigManagerApi;

  FindRemovableDrives;

  PollTimer:=TTimer.Create(nil);
  fPolling:=false;
  PollTimer.OnTimer:=OnTimer;
  fPollTimerInterval:=5000;
  PollTimer.Interval:=fPollTimerInterval;
end;

destructor TDriveEjector.Destroy;
begin
  PollTimer.free;
  SetLength(RemovableDrives, 0);
  
  UnloadConfigManagerApi;
  UnloadSetupApi;
  inherited;
end;

procedure TDriveEjector.FindRemovableDrives;
const
  MAX_DRIVES = 26;
var
  dwDriveMask: DWORD;
  DriveName: string;
  i: integer;
begin
  SetLength(RemovableDrives, 0);

  dwDriveMask := GetLogicalDrives;
  DriveName := 'A:\';
  for i := 0 to MAX_DRIVES - 1 do
  begin
    if (dwDriveMask and (1 shl I)) <> 0 then
    begin
      DriveName[1] := 'A';
      Inc(DriveName[1], I);

      // Check drive is a valid volume
      if GetDriveType(PChar(DriveName)) > 1 then
      begin  
        //Check if device is USB
        ScanDrive(DriveName[1]);
      end;
    end;
  end;

  //Finally check if any are card readers
  CheckForCardReaders;
end;

procedure TDriveEjector.ScanDrive(DriveLetter: Char);
type
  PCharArray = ^TCharArray;
  TCharArray = array[0..32767] of Char;

STORAGE_PROPERTY_QUERY = packed record
  PropertyId: DWORD;
  QueryType: DWORD;
  AdditionalParameters: array[0..3] of Byte;
end;

STORAGE_DEVICE_DESCRIPTOR = packed record
  Version: ULONG;
  Size: ULONG;
  DeviceType: Byte;
  DeviceTypeModifier: Byte;
  RemovableMedia: Boolean;
  CommandQueueing: Boolean;
  VendorIdOffset: ULONG;
  ProductIdOffset: ULONG;
  ProductRevisionOffset: ULONG;
  SerialNumberOffset: ULONG;
  STORAGE_BUS_TYPE: DWORD;
  RawPropertiesLength: ULONG;
  RawDeviceProperties: array[0..511] of Byte;
end;

const
  IOCTL_STORAGE_QUERY_PROPERTY = $2D1400;
var
  Returned, FFileHandle: Cardinal;
  Status: LongBool;
  PropQuery: STORAGE_PROPERTY_QUERY;
  DeviceDescriptor: STORAGE_DEVICE_DESCRIPTOR;
  PCh: PChar;
  Inst: integer;
begin
  FFileHandle:=INVALID_HANDLE_VALUE;
  try
    FFileHandle := CreateFile(
                     PChar('\\.\' + DriveLetter + ':'),
                     0,
                     FILE_SHARE_READ or FILE_SHARE_WRITE,
                     nil,
                     OPEN_EXISTING,
                     0,
                     0
                   );

    if FFileHandle = INVALID_HANDLE_VALUE then exit;

    ZeroMemory(@PropQuery, SizeOf(PropQuery));
    ZeroMemory(@DeviceDescriptor, SizeOf(DeviceDescriptor));
    DeviceDescriptor.Size := SizeOf(DeviceDescriptor);

    Status := DeviceIoControl(
                FFileHandle,
                IOCTL_STORAGE_QUERY_PROPERTY,
                @PropQuery,
                SizeOf(PropQuery),
                @DeviceDescriptor,
                DeviceDescriptor.Size,
                @Returned,
                nil
              );

    if not Status then exit;

    if DeviceDescriptor.STORAGE_BUS_TYPE <> 0 then
    begin
      if (DeviceDescriptor.STORAGE_BUS_TYPE = 7) or
         (DeviceDescriptor.STORAGE_BUS_TYPE = 4) then  //7 is USB, 4 is firewire

        SetLength(RemovableDrives, length(RemovableDrives) + 1) //enlarge the array for another item
      else
        exit;
    end
    else
      exit;

    //Drive Letter
    RemovableDrives[high(RemovableDrives)].DriveLetter := DriveLetter;

    //Volume Name
    RemovableDrives[high(RemovableDrives)].VolumeName :=Trim(GetVolumeName(DriveLetter));

    //Vendor Id
    if DeviceDescriptor.VendorIdOffset <> 0 then
    begin
      PCh := @PCharArray(@DeviceDescriptor)^[DeviceDescriptor.VendorIdOffset];
      RemovableDrives[high(RemovableDrives)].VendorId := Trim(PCh);
    end;

    //Product Id
    if DeviceDescriptor.ProductIdOffset <> 0 then
    begin
      PCh := @PCharArray(@DeviceDescriptor)^[DeviceDescriptor.ProductIdOffset];
      RemovableDrives[high(RemovableDrives)].ProductID := Trim(PCh);
    end;

    //Product Revision
    if DeviceDescriptor.ProductRevisionOffset <> 0 then
    begin
      PCh := @PCharArray(@DeviceDescriptor)^[DeviceDescriptor.ProductRevisionOffset];
      RemovableDrives[high(RemovableDrives)].ProductRevision := Trim(PCh);
    end;

    //Is Card Reader   //This is checked and changed later
    RemovableDrives[high(RemovableDrives)].IsCardReader := false;

    //Does Card Reader have media in it?
    if CheckIfDriveHasMedia(DriveLetter) then
      RemovableDrives[high(RemovableDrives)].CardMediaPresent:=true
    else
      RemovableDrives[high(RemovableDrives)].CardMediaPresent:=false;

    //Bus Type
    RemovableDrives[high(RemovableDrives)].BusType:= DeviceDescriptor.STORAGE_BUS_TYPE;

    //Parents Device Instance
    if GetParentDriveDevInst(DriveLetter, Inst) then
      RemovableDrives[high(RemovableDrives)].ParentDevInst:=Inst;

  finally
    if FFileHandle <> INVALID_HANDLE_VALUE then CloseHandle(FFileHandle);
  end;
end;

function TDriveEjector.GetCardPolling: boolean;
begin
  result:=fPolling;
end;

procedure TDriveEjector.SetCardPolling(Value: boolean);
begin
  fPolling:=Value;
  PollTimer.Enabled:=fPolling;
end;

function TDriveEjector.GetDrivesCount: integer;
begin
  result:=Length(RemovableDrives);
end;

//function TDriveEjector.RemoveDrive(DriveLetter: string; ShowEjectMessage: boolean = false; CardEject: boolean = false): boolean;
//var
//  DriveIndex, i: integer;
//begin
//  result:=false;
//  DriveIndex:=-1;
//
//  //First find the DriveLetter
//  if DrivesCount = 0 then exit;
//  for I := 0 to DrivesCount - 1 do
//  begin
//    if RemovableDrives[i].DriveLetter=DriveLetter then
//    begin
//      DriveIndex:=i;
//      break;
//    end;
//  end;
//
//  if DriveIndex <> -1 then
//  begin
//    if CardEject then
//    begin
//      if EjectCard(DriveLetter, i) then //keep the card reader device
//      begin
//        RemovableDrives[i].CardMediaPresent:=false;
//        result:=true;
//      end;
//    end
//    else
//    if EjectDevice(DriveLetter, i, ShowEjectMessage) then
//    begin
//      DeleteArrayItem(DriveIndex);
//      result:=true;
//    end;
//  end;
//
//end;

//This version returns an error code on failures
function TDriveEjector.RemoveDrive(DriveLetter: string;
  var EjectErrorCode: integer; ShowEjectMessage, CardEject, CloseRunningApps, ForceRunningAppsClosure: boolean): boolean;
var
  DriveIndex, i: integer;
begin
  result:=false;
  EjectErrorCode:=REMOVE_ERROR_NONE;
  DriveIndex:=-1;

  //First find the DriveLetter
  if DrivesCount = 0 then
  begin
    EjectErrorCode:=REMOVE_ERROR_DRIVE_NOT_FOUND;
    exit;
  end;

  for I := 0 to DrivesCount - 1 do
  begin
    if RemovableDrives[i].DriveLetter=DriveLetter then
    begin
      DriveIndex:=i;
      break;
    end;
  end;

  if DriveIndex <> -1 then
  begin
    //First try and close explorer windows
    EnumWindows(@EnumWindowsAndCloseFunc, LParam(DriveLetter[1]));
    //Then try and close any programs that might be running from the drive
    if CloseRunningApps then
      CloseAppsRunningFrom(DriveLetter[1], ForceRunningAppsClosure);


    //CHECK - stop card style eject if device isnt a card
    if RemovableDrives[DriveIndex].IsCardReader = false then
      CardEject:=false;

    if CardEject then //keep the card reader device - eject the media
    begin
      if EjectCard(DriveLetter, EjectErrorCode) then
      begin
        RemovableDrives[i].CardMediaPresent:=false;
        result:=true
      end;
    end
    else
    if EjectDevice(DriveLetter, EjectErrorCode, ShowEjectMessage) then
    begin
      DeleteArrayItem(DriveIndex);
      result:=true;
    end;
  end
  else
    EjectErrorCode:=REMOVE_ERROR_DRIVE_NOT_FOUND;

end;

procedure TDriveEjector.RescanAllDrives;
begin
  FindRemovableDrives;
end;

procedure TDriveEjector.CheckForCardReaders;
var
  i: integer;
begin
  if DrivesCount = 0 then exit;

  for i := 0 to DrivesCount - 1 do
  begin
    if GetNoDevicesWithSameParentInst(RemovableDrives[i].ParentDevInst) > 1 then
      if GetNoDevicesWithSameProductID(RemovableDrives[i].ProductId) > 1 then //Hard drive partitions
        RemovableDrives[i].IsCardReader:=False
      else
        RemovableDrives[i].IsCardReader:=True //Matching devices with parent inst but differing device names are likely to be card readers
  end;
end;

function TDriveEjector.CheckIfDriveHasMedia(DriveLetter: string): boolean;
var
  Returned, DriveHandle: cardinal;
begin
  result:=false;
                                                               {GENERIC_READ or GENERIC_WRITE}
  DriveHandle := CreateFile(PChar('\\.\' + DriveLetter + ':'), FILE_SHARE_READ or FILE_SHARE_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  try
    if DeviceIoControl(DriveHandle, IOCTL_STORAGE_CHECK_VERIFY, nil, 0, nil, 0, @Returned, nil) then
      result:=true; //Card is in reader

  finally
    CloseHandle(Drivehandle);
  end;
end;

function TDriveEjector.EjectCard(DriveLetter: string; var EjectErrorCode: integer): boolean;
var
  Returned, DriveHandle: cardinal;
begin
  result:=false;
                                                                {GENERIC_READ or GENERIC_WRITE}
  DriveHandle := CreateFile(PChar('\\.\' + DriveLetter + ':'), FILE_SHARE_READ or FILE_SHARE_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  try
    if DriveHandle = INVALID_HANDLE_VALUE then
    begin
      if GetLastError = 32 then
        EjectErrorCode:=REMOVE_ERROR_DISK_IN_USE
      else
        EjectErrorCode:=REMOVE_ERROR_UNKNOWN_ERROR;

      exit;
    end;

    if DeviceIoControl(DriveHandle, IOCTL_STORAGE_CHECK_VERIFY, nil, 0, nil, 0, @Returned, nil) = false then
    begin
      EjectErrorCode:=REMOVE_ERROR_NO_CARD_MEDIA;
      exit; //No card in reader
    end;


    result:=DeviceIoControl(Drivehandle, IOCTL_STORAGE_EJECT_MEDIA, nil, 0, nil, 0, @Returned, nil);

    if result=false then
    begin
      if GetLastError = 32 then
        EjectErrorCode:=REMOVE_ERROR_DISK_IN_USE
      else
        EjectErrorCode:=REMOVE_ERROR_UNKNOWN_ERROR;
    end;
    
  finally
    CloseHandle(Drivehandle);
  end;
end;

function TDriveEjector.EjectDevice(DriveLetter: string; var EjectErrorCode: integer; ShowEjectMessage: boolean = false): boolean;
var
  szRootPath, szDevicePath, szVolumeAccessPath: string;
  DeviceNumber: longint;
  dwBytesReturned: DWord;
  DriveType: UINT;
  hVolume: THandle;
  SDN: STORAGE_DEVICE_NUMBER;
  funcResult, tries: integer;
  funcResultBool: boolean;
  DeviceInst, DevInstParent: DEVINST;
  szDosDeviceName: array[0..MAX_PATH-1] of Char;
  VetoType: PNP_VETO_TYPE;
  VetoNameW: array[0..MAX_PATH-1] of WCHAR;
begin
  Result:=false;
  szRootPath:=DriveLetter + ':';
  szDevicePath:=DriveLetter +  ':';
  szVolumeAccessPath:='\\.\' + DriveLetter + ':';
  DeviceNumber:=-1;

  hVolume:=INVALID_HANDLE_VALUE;
  try
    //Open the storage volume
    hVolume:=CreateFile(PChar(szVolumeAccessPath), 0, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
    if hVolume = INVALID_HANDLE_VALUE then
    begin
      if GetLastError = 32 then
        EjectErrorCode:=REMOVE_ERROR_DISK_IN_USE
      else
        EjectErrorCode:=REMOVE_ERROR_UNKNOWN_ERROR;

      exit;
    end;


    //Get the volume's device number
    dwBytesReturned:=0;
    funcResultBool:=DeviceIoControl(hVolume, IOCTL_STORAGE_GET_DEVICE_NUMBER, nil, 0, @SDN, SizeOf(SDN), @dwBytesReturned, nil);
    if funcResultBool = true  then
      DeviceNumber:=SDN.DeviceNumber;

  finally
    CloseHandle(hVolume);
  end;

  if DeviceNumber = -1 then
  begin
    EjectErrorCode:=REMOVE_ERROR_WINAPI_ERROR;
    exit;
  end;
    

	//Get the drive type
	DriveType := GetDriveType(PChar(szRootPath));
  szDosDeviceName[0]:=#0;

	//Get the dos device name (like \deviceloppy0) to decide if it's a floppy or not
	funcResult := QueryDosDevice(PChar(szDevicePath), szDosDeviceName,  MAX_PATH);
	if funcResult = 0 then
  begin
    EjectErrorCode:=REMOVE_ERROR_WINAPI_ERROR;
    exit;
  end;
    

	//Get the device instance handle of the storage volume by means of a SetupDi enum and matching the device number
	DeviceInst:= GetDrivesDevInstByDeviceNumber(DeviceNumber, DriveType, szDosDeviceName);
	if ( DeviceInst = 0 ) then
  begin
    EjectErrorCode:=REMOVE_ERROR_WINAPI_ERROR;
    exit;
  end;
    

	VetoType := PNP_VetoTypeUnknown;
	VetoNameW[0] := #0;

	//Get drives's parent - this is what gets ejected
	DevInstParent := 0;
	CM_Get_Parent(DevInstParent, DeviceInst, 0);

  //Try and eject 3 times
  for tries := 0 to 2 do
  begin
		VetoNameW[0] := #0;

    if ShowEjectMessage then
      funcResult := CM_Request_Device_EjectW(DevInstParent, nil, nil, 0, 0) //With messagebox (W2K, Vista) or balloon (XP)
    else
		  funcResult := CM_Request_Device_EjectW(DevInstParent, @VetoType, VetoNameW, MAX_PATH, 0);

		if (funcResult=CR_SUCCESS) and (VetoType=PNP_VetoTypeUnknown) then
    begin
      Result:=true;
			break;
    end;

		Sleep(500); //Wait and then try again
	 end;

   if result=false then
   begin
    //if GetLastError = 32 then
      EjectErrorCode:=REMOVE_ERROR_DISK_IN_USE
    //else
   //   EjectErrorCode:=REMOVE_ERROR_UNKNOWN_ERROR;
   end;
end;

function TDriveEjector.GetDrivesDevInstByDeviceNumber(DeviceNumber: Integer;
  DriveType: UINT; szDosDeviceName: PCHAR): DEVINST;
var
  IsFloppy, DoLoop: boolean;
  myGUID: TGUID;
  myhDevInfo: HDEVINFO;
  dwIndex, dwSize, dwBytesReturned: DWORD;
  //Buf: array[0..1024-1] of BYTE;
  FunctionResult: boolean;
  pspdidd: PSPDeviceInterfaceDetailData;
	spdid: SP_DEVICE_INTERFACE_DATA;
	spdd: SP_DEVINFO_DATA;
  hDrive: THandle;
  SDN: STORAGE_DEVICE_NUMBER;
begin
  Result:=0;
  IsFloppy:=true;

  if StrPos(szDosDeviceName, '\Floppy') = nil then
    IsFloppy:=false;

	case (DriveType)  of
    DRIVE_REMOVABLE:
		  if ( IsFloppy ) then
      begin
			  myguid := GUID_DEVINTERFACE_FLOPPY;
      end
      else
      begin
			  myguid := GUID_DEVINTERFACE_DISK;
      end;

    DRIVE_FIXED:
		  myguid := GUID_DEVINTERFACE_DISK;

    DRIVE_CDROM:
		  myguid := GUID_DEVINTERFACE_CDROM;

    else
      exit;

  end;

	//Get device interface info set handle for all devices attached to system
	myhDevInfo := SetupDiGetClassDevs(@myguid, nil, 0, DIGCF_PRESENT or DIGCF_DEVICEINTERFACE);

	if (cardinal(myhDevInfo) = INVALID_HANDLE_VALUE) then
    exit;

	//Retrieve a context structure for a device interface of a device information set
	dwIndex := 0;

	//pspdidd :=PSP_DEVICE_INTERFACE_DETAIL_DATA(@Buf);
  ZeroMemory(@spdd, SizeOf(spdd));
	spdid.cbSize := SizeOf(spdid);

  DoLoop:=True;
	while (DoLoop=true)	do
  begin
		FunctionResult := SetupDiEnumDeviceInterfaces(myhDevInfo, nil, myGUID, dwIndex, spdid);
		if FunctionResult= false then
			break;

		dwSize := 0;
		SetupDiGetDeviceInterfaceDetail(myhDevInfo, @spdid, nil, 0, dwSize, nil); //Check the buffer size

		if ( dwSize <> 0)  and  (dwSize <= 1024) {SizeOf(Buf))} then
    begin
      GetMem(pspdidd, dwSize);
      try
			  pspdidd.cbSize := SizeOf(pspdidd^); //SizeOf(TSPDeviceInterfaceDetailData)
			  ZeroMemory(@spdd, SizeOf(spdd));
			  spdd.cbSize := SizeOf(spdd);

			  FunctionResult := SetupDiGetDeviceInterfaceDetail(myhDevInfo, @spdid, pspdidd, dwSize, dwSize, @spdd);
			  if FunctionResult then
        begin
          //Open the disk or cdrom or floppy
          hDrive:=INVALID_HANDLE_VALUE;
          try
            hDrive := CreateFile(pspdidd.DevicePath, 0, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
				    if ( hDrive <> INVALID_HANDLE_VALUE ) then
            begin
					    //Get its device number
					    dwBytesReturned := 0;
					    FunctionResult := DeviceIoControl(hDrive, IOCTL_STORAGE_GET_DEVICE_NUMBER, nil, 0, @sdn, SizeOf(sdn), @dwBytesReturned, nil);
					    if FunctionResult  then
              begin
						    if DeviceNumber = LongInt(sdn.DeviceNumber) then
                begin  //Match the device number with that of the current device
							    result:= spdd.DevInst;
                  break;
						    end;
				      end;
            end;
          finally
            CloseHandle(hDrive);
          end;
			  end;

      finally
        FreeMem(pspdidd);
      end;
	  end;

    dwIndex:= dwIndex + 1;
  end;

	SetupDiDestroyDeviceInfoList(myhDevInfo);
end;

function TDriveEjector.GetNoDevicesWithSameParentInst(
  ParentDevInst: integer): integer;
var
  i: integer;
begin
  result:=-1; //will be inc'ed once when it goes through the one we're comparing to
  for I := 0 to DrivesCount - 1 do
  begin
    if RemovableDrives[i].ParentDevInst = ParentDevInst then
      inc(result);
  end;
end;

function TDriveEjector.GetNoDevicesWithSameProductId(
  ProductId: string): integer;
var
  i: integer;
begin
  result:=-1; //will be inc'ed once when it goes through the one we're comparing to
  for I := 0 to DrivesCount - 1 do
  begin
    if RemovableDrives[i].ProductID = ProductID then
      inc(result);
  end;
end;

function TDriveEjector.GetParentDriveDevInst(DriveLetter: string;
  var ParentInstNum: integer): boolean;
var
  szRootPath, szDevicePath, szVolumeAccessPath: string;
  DeviceNumber: longint;
  hVolume: THandle;
  dwBytesReturned: DWord;
  DriveType: UINT;
  SDN: STORAGE_DEVICE_NUMBER;
  FunctionResultInt: integer;
  FunctionResultBool: boolean;
  DeviceInst, DevInstParent: DEVINST;
  szDosDeviceName: array[0..MAX_PATH-1] of Char;
begin
  Result:=false;
  szRootPath:=DriveLetter + ':';
  szDevicePath:=DriveLetter +  ':';
  szVolumeAccessPath:='\\.\' + DriveLetter + ':';
  DeviceNumber:=-1;

  hVolume:= INVALID_HANDLE_VALUE;
  try
    //Open the storage volume
    hVolume:=CreateFile(PChar(szVolumeAccessPath), 0, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
    if hVolume = INVALID_HANDLE_VALUE then
      exit;

    //Get the volume's device number
    dwBytesReturned:=0;
    FunctionResultBool:=DeviceIoControl(hVolume, IOCTL_STORAGE_GET_DEVICE_NUMBER, nil, 0, @SDN, SizeOf(SDN), @dwBytesReturned, nil);
    if FunctionResultBool then
      DeviceNumber:=SDN.DeviceNumber;

  finally
    CloseHandle(hVolume);
  end;

  if DeviceNumber = -1 then
    exit;

	//Get the drive type which is required to match the device numbers correctely
	DriveType := GetDriveType(PChar(szRootPath));
  szDosDeviceName[0]:=#0;

	//Get the dos device name (like \deviceloppy0) to decide if it's a floppy or not
	FunctionResultInt := QueryDosDevice(PChar(szDevicePath), szDosDeviceName,  MAX_PATH);
	if FunctionResultInt = 0 then
    exit;

	//Get the device instance handle of the storage volume by means of a SetupDi enum and matching the device number
	DeviceInst:= GetDrivesDevInstByDeviceNumber(DeviceNumber, DriveType, szDosDeviceName);

	if (DeviceInst = 0) then
    exit;

	//Get drives's parent
	DevInstParent:=0;
	CM_Get_Parent(DevInstParent, DeviceInst, 0);

  if DevInstParent > 0 then
  begin
    ParentInstNum:=DevInstParent;
    result:=true;
  end;

end;

procedure TDriveEjector.OnTimer(Sender: TObject);
var
  i: integer;
begin
  //sysutils.Beep;
  if GetDrivesCount = 0 then exit;

  for I := 0 to GetDrivesCount - 1 do
  begin
    if RemovableDrives[i].IsCardReader then
    begin
      if CheckIfDriveHasMedia(RemovableDrives[i].DriveLetter) then
      begin
        if RemovableDrives[i].CardMediaPresent=false then //Has changed - generate event
          if assigned(foncardmediachanged) then
            foncardmediachanged(nil);

        RemovableDrives[i].CardMediaPresent:=true
      end
      else
      begin
        if RemovableDrives[i].CardMediaPresent=true then  //Has changed - generate event
          if assigned(foncardmediachanged) then
            foncardmediachanged(nil);

        RemovableDrives[i].CardMediaPresent:=false
      end;
    end;
  end;

end;

end.
